unit ParentEventUnit;

interface

uses
  Classes, ParentMainFormUnit, Windows, SyncObjs;

type
  TTestParentEvent = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on the event having happened
    procedure CodeReliantOnTheEvent;
  public
    constructor Create;
    destructor Destroy; override;
  end;

var
  Event: TEvent;

const
  ManualResetFlag = False;

implementation

uses
  SysUtils;

const
  EventName = 'SampleEvent';

constructor TTestParentEvent.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestParentEvent.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestParentEvent.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestParentEvent.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestParentEvent.Execute;
begin
  WriteString(Format(
    'Thread %x waiting for event', [ThreadID]));
  if Event.WaitFor(Infinite) = wrSignaled then
    CodeReliantOnTheEvent;
end;

procedure TTestParentEvent.CodeReliantOnTheEvent;
begin
  WriteString(Format(
    'Thread %x has seen the event', [ThreadID]));
end;

initialization
  //Auto-reset event:
  //  PulseEvent signals event, lets one waiting thread
  //  through if there is one, and sets it to non-signaled.
  //  SetEvent signals the event until one waiting thread
  //  goes through, and then sets it to unsignaled.
  //Manual reset event:
  //  PulseEvent signals event, lets all waiting
  //  threads through, and sets it to non-signaled.
  //  SetEvent signals event, letting any waiting
  //  threads through and leaves it signalled.
  //  ResetEvent sets event to nonsignaled.
  Event := TEvent.Create(nil,
    ManualResetFlag, //False = auto-reset, True = manual-reset
    False, EventName)
finalization
  Event.Free
end.
