unit ChildSemaphoreUnit;

interface

uses
  SysUtils, Classes, ChildMainFormUnit;

type
  TTestChildSemaphore = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //Can only execute 3 simultaneous times
    procedure Max3SimultaneousExecutions;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  Windows;

const
  SemaphoreName = 'SampleSemaphore';

constructor TTestChildSemaphore.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestChildSemaphore.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestChildSemaphore.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestChildSemaphore.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestChildSemaphore.Execute;
const
  //Not defined in Windows.Pas
  Semaphore_All_Access =
    Standard_Rights_Required or Windows.Synchronize or 3;
var
  LocalSemaphore: THandle;
  SemaphoreCount: Integer;
begin
  LocalSemaphore := OpenSemaphore(Semaphore_All_Access, False, SemaphoreName);
  if LocalSemaphore = 0 then
  begin
    WriteString('Semaphore not found');
    Exit
  end;
  WriteString(Format(
    'Thread %x waiting for Semaphore', [ThreadID]));
  WaitForSingleObject(LocalSemaphore, Infinite);
  Max3SimultaneousExecutions;
  ReleaseSemaphore(LocalSemaphore, 1, @SemaphoreCount);
  CloseHandle(LocalSemaphore);
  WriteString(Format(
    'Thread %x released semaphore whose count was %d',
    [ThreadID, SemaphoreCount]));
end;

procedure TTestChildSemaphore.Max3SimultaneousExecutions;
begin
  WriteString(Format(
    'Thread %x got Semaphore', [ThreadID]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with Semaphore', [ThreadID]));
end;

end.
