unit SemaphoreUnit;

interface

uses
  Classes, MainFormUnit;

type
  TTestSemaphore = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //Can only execute 3 simultaneous times
    procedure Max3SimultaneousExecutions;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Windows;

var
  Semaphore: THandle;

const
  SemaphoreName = 'SampleSemaphore';
  MaxSemaphoreCount = 3;

constructor TTestSemaphore.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [Handle]))
end;

destructor TTestSemaphore.Destroy;
begin
  WriteString(Format('Thread %x terminated', [Handle]));
  inherited Destroy
end;

procedure TTestSemaphore.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestSemaphore.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestSemaphore.Execute;
const
  //Not defined in Windows.Pas
  Semaphore_All_Access =
    Standard_Rights_Required or Windows.Synchronize or 3;
var
  SemaphoreCount: Integer;
begin
  WriteString(Format(
    'Thread %x waiting for semaphore', [Handle]));
  WaitForSingleObject(Semaphore, Infinite);
  Max3SimultaneousExecutions;
  ReleaseSemaphore(Semaphore, 1, @SemaphoreCount);
  WriteString(Format(
    'Thread %x released semaphore whose count was %d',
    [Handle, SemaphoreCount]));
end;

procedure TTestSemaphore.Max3SimultaneousExecutions;
begin
  WriteString(Format(
    'Thread %x got semaphore', [Handle]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with semaphore', [Handle]));
end;

initialization
  Semaphore := CreateSemaphore(
    nil, MaxSemaphoreCount, MaxSemaphoreCount, SemaphoreName)
finalization
  //This is not strictly necessary as all open
  //handles are closed by the Win32 OS upon termination
  if Semaphore <> 0 then
    CloseHandle(Semaphore)
end.