unit CopyU2;

interface

type
   TCallBack = procedure (Position, Size: LongInt);

procedure FileCopy(const InFileName, OutFileName: String;
  CallBack: TCallBack);

implementation

uses
  SysUtils;

procedure FileCopy(const InFileName, OutFileName: String;
  CallBack: TCallBack);
const
  BufSize = 8 * 4096; { 32kb }
type
  PBuffer = ^TBuffer;
  TBuffer = array[1..BufSize] of Byte;
var
  Size: Cardinal;
  Buffer: PBuffer;
  InFile, OutFile: File;
  SizeDone, SizeFile: Longint;
begin
  if InFileName = OutFileName then
    raise EInOutError.Create('File cannot be copied onto itself')
  else
  begin
    Buffer := nil;
    Assign(InFile, InFileName);
    Reset(InFile, 1);
    try
      SizeFile := FileSize(InFile);
      Assign(OutFile, OutFileName);
      Rewrite(OutFile, 1);
      try
        SizeDone := 0;
        New(Buffer);
        repeat
          BlockRead(InFile, Buffer^, BufSize, Size);
          Inc(SizeDone, Size);
          if Assigned(@CallBack) then
            CallBack(SizeDone, SizeFile);
          BlockWrite(OutFile, Buffer^, Size)
        until Size < BufSize;
        FileSetDate(TFileRec(OutFile).Handle,
          FileGetDate(TFileRec(InFile).Handle));
      finally
        if Buffer <> nil then
          Dispose(Buffer);
        CloseFile(OutFile)
      end;
    finally
      CloseFile(InFile);
    end
  end
end;

end.

